@@ -102,11 +102,15 @@ static int first_last_add_cb (EBookBackendFileIndex *index, EContact *contact,
     EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
 static int last_first_add_cb (EBookBackendFileIndex *index, EContact *contact,
     EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
+static int nickname_add_cb (EBookBackendFileIndex *index, EContact *contact,
+    EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
 
 static int first_last_remove_cb (EBookBackendFileIndex *index, EContact *contact,
     EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
 static int last_first_remove_cb (EBookBackendFileIndex *index, EContact *contact,
     EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
+static int nickname_remove_cb (EBookBackendFileIndex *index, EContact *contact,
+    EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops);
 
 static int lexical_ordering_cb (DB *secondary, const DBT *akey, const DBT *bkey);
 
@@ -117,6 +121,7 @@ static const EBookBackendFileIndexData indexes[] = {
   {"email", "email", FALSE, NULL, NULL, EVC_EMAIL, NULL},
   {"first-last", "first_last", FALSE, first_last_add_cb, first_last_remove_cb, NULL, lexical_ordering_cb},
   {"last-first", "last_first", FALSE, last_first_add_cb, last_first_remove_cb, NULL, lexical_ordering_cb},
+  {"nick", "nick", FALSE, nickname_add_cb, nickname_remove_cb, EVC_NICKNAME, lexical_ordering_cb}
 };
 
 typedef struct _EBookBackendFileIndexPrivate EBookBackendFileIndexPrivate;
@@ -370,6 +375,7 @@ e_book_backend_file_index_setup_indicies (EBookBackendFileIndex *index, DB *db,
   int i = 0;
   DB *sdb = NULL;
   DB_ENV *env = NULL;
+  gboolean need_cleanup;
 
   /* for each database we go through and try and create it exclusively, if
    * that fails then we know that we don't need to populate it so we go and
@@ -384,6 +390,8 @@ e_book_backend_file_index_setup_indicies (EBookBackendFileIndex *index, DB *db,
     return FALSE;
   }
 
+  need_cleanup = indices_need_cleanup (env);
+
   for (i = 0; i < G_N_ELEMENTS (indexes); i++)
   {
     char *index_db_path;
@@ -395,6 +403,13 @@ e_book_backend_file_index_setup_indicies (EBookBackendFileIndex *index, DB *db,
 
     index_db_path = make_index_db_path (index_dirname, indexes[i].index_name);
 
+    if (need_cleanup) {
+      /* The indices are in an inconsistent state, so we need to clean them
+       * up and regenerate them */
+      DEBUG ("cleaning up index file: %s", index_db_path);
+      g_unlink (index_db_path);
+    }
+
     /* we cannot pass DB_CREATE in the first time, because if index dbs aren't
      * present we need to populate them */
     db_error = sdb->open (sdb, NULL, index_db_path, NULL, DB_BTREE,
@@ -432,6 +447,9 @@ e_book_backend_file_index_setup_indicies (EBookBackendFileIndex *index, DB *db,
     g_hash_table_insert (priv->sdbs, indexes[i].query_term, sdb);
   }
 
+  if (need_cleanup)
+    indices_mark_as_clean (env);
+
   /* now go through and populate these */
   if (dbs_to_populate)
   {
@@ -859,8 +877,17 @@ static char *key_priority[] = {
   NULL
 };
 
+static char *key_priority_nick[] = {
+  EVC_NICKNAME,
+  EVC_N,
+  EVC_X_COMPANY,
+  EVC_TEL,
+  EVC_EMAIL,
+  NULL
+};
+
 static EVCardAttribute *
-get_key_attribute (EContact *contact)
+get_key_attribute (EContact *contact, const char **priority)
 {
   GList *attrs = NULL;
   GHashTable *ht = NULL;
@@ -882,7 +909,7 @@ get_key_attribute (EContact *contact)
 
     DEBUG ("looking attrib '%s'", attr_name);
 
-    if (g_str_equal (attr_name, key_priority[0])) {
+    if (g_str_equal (attr_name, priority[0])) {
       if (!e_vcard_attribute_get_values (attr))
         continue;
 
@@ -894,8 +921,8 @@ get_key_attribute (EContact *contact)
   }
 
   /* iterate from 2nd element since 1st was not found */
-  for (i = 1; key_priority[i]; i++) {
-    key_attr = (EVCardAttribute *) g_hash_table_lookup (ht, key_priority[i]);
+  for (i = 1; priority[i]; i++) {
+    key_attr = (EVCardAttribute *) g_hash_table_lookup (ht, priority[i]);
     if (key_attr && e_vcard_attribute_get_values (key_attr)) {
       break;
     }
@@ -922,7 +949,8 @@ get_key_attribute (EContact *contact)
 enum
 {
   ORDER_FIRSTLAST = 0,
-  ORDER_LASTFIRST
+  ORDER_LASTFIRST,
+  ORDER_NICKNAME
 };
 
 static char *
@@ -935,7 +963,12 @@ get_key (EContact *contact, gint ordering)
 
   g_return_val_if_fail (E_IS_CONTACT (contact), NULL);
 
-  key_attr = get_key_attribute (contact);
+  if (ordering == ORDER_NICKNAME) {
+    key_attr = get_key_attribute (contact, (const char **) key_priority_nick);
+  }
+  else {
+    key_attr = get_key_attribute (contact, (const char **) key_priority);
+  }
   if (!key_attr) {
     WARNING ("contact is empty?");
     return NULL;
@@ -1002,7 +1035,7 @@ generic_name_remove_cb (EBookBackendFileIndex *index, EContact *contact,
 
   DEBUG ("removing from index with key %s and data %s", key, uid);
 
-  db_error = txn_ops_add_new (ops, TXN_CDEL, db, key, g_strdup (uid));
+  db_error = txn_ops_add_new (ops, TXN_INDEX_DEL, db, key, g_strdup (uid));
   if (db_error != 0) {
     WARNING ("cannot create new txn item: %s", db_strerror (db_error));
   }
@@ -1038,6 +1071,20 @@ last_first_remove_cb (EBookBackendFileIndex *index, EContact *contact,
   return generic_name_remove_cb (index, contact, data, db, uid, ORDER_LASTFIRST, ops);
 }
 
+static int
+nickname_add_cb (EBookBackendFileIndex *index, EContact *contact,
+    EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops)
+{
+  return generic_name_add_cb (index, contact, data, db, uid, ORDER_NICKNAME, ops);
+}
+
+static int
+nickname_remove_cb (EBookBackendFileIndex *index, EContact *contact,
+    EBookBackendFileIndexData *data, DB *db, const gchar *uid, GPtrArray *ops)
+{
+  return generic_name_remove_cb (index, contact, data, db, uid, ORDER_NICKNAME, ops);
+}
+
 /*
  * generic version of function to get the index key from a contact based on
  * the VCard field
@@ -1166,7 +1213,7 @@ generic_field_remove (EBookBackendFileIndex *index, EContact *contact,
     DEBUG ("removing from index '%s' with key %s and data %s",
            data->index_name, key, uid);
 
-    db_error = txn_ops_add_new (ops, TXN_CDEL, db, g_strdup (key), g_strdup (uid));
+    db_error = txn_ops_add_new (ops, TXN_INDEX_DEL, db, g_strdup (key), g_strdup (uid));
     if (db_error != 0) {
       WARNING ("cannot create new txn item: %s", db_strerror (db_error));
       return db_error;
